@@ -95,7 +95,6 @@ pycairo_get_hairline (PycairoContext *o, PyObject *ignored) {
 }
 #endif
 
-#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 15, 10)
 static PyObject *
 pycairo_tag_begin (PycairoContext *o, PyObject *args) {
     const char *tag_name;
@@ -135,7 +134,6 @@ pycairo_tag_end (PycairoContext *o, PyObject *args) {
       RETURN_NULL_IF_CAIRO_CONTEXT_ERROR(o->ctx);
       Py_RETURN_NONE;
 }
-#endif
 
 static PyObject *
 pycairo_append_path (PycairoContext *o, PyObject *args) {
@@ -389,7 +387,7 @@ pycairo_get_dash (PycairoContext *o, PyObject *ignored) {
     PyErr_SetString (PyExc_RuntimeError, "invalid dash return");
     return NULL;
   }
-  dashes = PyMem_Malloc ((unsigned int)count * sizeof(double));
+  dashes = PyMem_Calloc ((unsigned int)count, sizeof(double));
   if (dashes == NULL)
     return PyErr_NoMemory();
 
@@ -516,6 +514,12 @@ pycairo_glyph_extents (PycairoContext *o, PyObject *args) {
 			 &py_object, &num_glyphs))
     return NULL;
 
+  if (PyTuple_Size (args) > 1) {
+    PyErr_WarnEx (PyExc_DeprecationWarning,
+                  "The num_glyphs parameter to Context.glyph_extents is deprecated",
+                  1);
+  }
+
   glyphs = _PycairoGlyphs_AsGlyphs (py_object, &num_glyphs);
   if (glyphs == NULL)
     return NULL;
@@ -540,6 +544,12 @@ pycairo_glyph_path (PycairoContext *o, PyObject *args) {
 			 &py_object, &num_glyphs))
     return NULL;
 
+  if (PyTuple_Size (args) > 1) {
+    PyErr_WarnEx (PyExc_DeprecationWarning,
+                  "The num_glyphs parameter to Context.glyph_path is deprecated",
+                  1);
+  }
+
   glyphs = _PycairoGlyphs_AsGlyphs (py_object, &num_glyphs);
   if (glyphs == NULL)
     return NULL;
@@ -876,7 +886,7 @@ pycairo_set_dash (PycairoContext *o, PyObject *args) {
       PyErr_SetString (PyExc_ValueError, "dash sequence too large");
       return NULL;
   }
-  dashes = PyMem_Malloc ((unsigned int)num_dashes * sizeof(double));
+  dashes = PyMem_Calloc ((unsigned int)num_dashes, sizeof(double));
   if (dashes == NULL) {
     Py_DECREF(py_dashes);
     return PyErr_NoMemory();
@@ -1134,6 +1144,12 @@ pycairo_show_glyphs (PycairoContext *o, PyObject *args) {
 			 &py_object, &num_glyphs))
     return NULL;
 
+  if (PyTuple_Size (args) > 1) {
+    PyErr_WarnEx (PyExc_DeprecationWarning,
+                  "The num_glyphs parameter to Context.show_glyphs is deprecated",
+                  1);
+  }
+
   glyphs = _PycairoGlyphs_AsGlyphs (py_object, &num_glyphs);
   if (glyphs == NULL)
     return NULL;
@@ -1376,10 +1392,8 @@ static PyMethodDef pycairo_methods[] = {
   {"set_hairline",    (PyCFunction)pycairo_set_hairline,     METH_VARARGS},
   {"get_hairline",    (PyCFunction)pycairo_get_hairline,     METH_NOARGS},
 #endif
-#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 15, 10)
   {"tag_begin",       (PyCFunction)pycairo_tag_begin,        METH_VARARGS},
   {"tag_end",         (PyCFunction)pycairo_tag_end,          METH_VARARGS},
-#endif
   {"append_path",     (PyCFunction)pycairo_append_path,      METH_VARARGS},
   {"arc",             (PyCFunction)pycairo_arc,              METH_VARARGS},
   {"arc_negative",    (PyCFunction)pycairo_arc_negative,     METH_VARARGS},
