@@ -59,7 +59,7 @@ _PycairoGlyphs_AsGlyphs (PyObject *py_object, int *num_glyphs)
     if (*num_glyphs < 0 || *num_glyphs > length)
         *num_glyphs = (int)length;
 
-    glyphs = PyMem_Malloc ((unsigned int)*num_glyphs * sizeof(cairo_glyph_t));
+    glyphs = PyMem_Calloc ((unsigned int)*num_glyphs, sizeof(cairo_glyph_t));
     if (glyphs == NULL) {
         PyErr_NoMemory();
         goto error;
@@ -75,7 +75,7 @@ _PycairoGlyphs_AsGlyphs (PyObject *py_object, int *num_glyphs)
                              "each glyph item must be an (i,x,y) sequence");
             goto error;
         }
-        index = PYCAIRO_PyLong_AsLong (PySequence_Fast_GET_ITEM (py_seq, 0));
+        index = PyLong_AsLong (PySequence_Fast_GET_ITEM (py_seq, 0));
         if (PyErr_Occurred ())
             goto error;
         glyph->index = (unsigned long)index;
@@ -106,7 +106,7 @@ _PyGlyph_AsGlyph (PyObject *pyobj, cairo_glyph_t *glyph) {
         return -1;
     }
 
-    index = PYCAIRO_PyLong_AsLong (PySequence_Fast_GET_ITEM (pyobj, 0));
+    index = PyLong_AsLong (PySequence_Fast_GET_ITEM (pyobj, 0));
     if (PyErr_Occurred ())
         return -1;
     if (index < 0) {
@@ -146,20 +146,47 @@ static PyObject*
 glyph_repr(PyObject *self) {
     PyObject *format, *result;
 
-    format = PYCAIRO_PyUnicode_FromString (
+    format = PyUnicode_FromString (
         "cairo.Glyph(index=%r, x=%r, y=%r)");
     if (format == NULL)
         return NULL;
-    result = PYCAIRO_PyUnicode_Format (format, self);
+    result = PyUnicode_Format (format, self);
     Py_DECREF (format);
     return result;
 }
 
-static PyObject*
-glyph_getattro (PyObject *self, PyObject *name) {
-    return Pycairo_tuple_getattro (self, KWDS, name);
+static PyObject *
+glyph_get_index(PyObject *self, void *closure)
+{
+    PyObject *obj = PyTuple_GetItem (self, 0);
+    Py_XINCREF (obj);
+    return obj;
 }
 
+static PyObject *
+glyph_get_x(PyObject *self, void *closure)
+{
+    PyObject *obj = PyTuple_GetItem (self, 1);
+    Py_XINCREF (obj);
+    return obj;
+}
+
+static PyObject *
+glyph_get_y(PyObject *self, void *closure)
+{
+    PyObject *obj = PyTuple_GetItem (self, 2);
+    Py_XINCREF (obj);
+    return obj;
+}
+
+static PyGetSetDef glyph_getset[] = {
+    {"index", (getter)glyph_get_index},
+    {"x", (getter)glyph_get_x},
+    {"y", (getter)glyph_get_y},
+    {NULL,},
+};
+
+
 PyTypeObject PycairoGlyph_Type = {
     PyVarObject_HEAD_INIT(NULL, 0)
     "cairo.Glyph",                      /* tp_name */
@@ -177,7 +204,7 @@ PyTypeObject PycairoGlyph_Type = {
     0,                                  /* tp_hash */
     0,                                  /* tp_call */
     0,                                  /* tp_str */
-    glyph_getattro,                     /* tp_getattro */
+    0,                                  /* tp_getattro */
     0,                                  /* tp_setattro */
     0,                                  /* tp_as_buffer */
     Py_TPFLAGS_DEFAULT,                 /* tp_flags */
@@ -190,7 +217,7 @@ PyTypeObject PycairoGlyph_Type = {
     0,                                  /* tp_iternext */
     0,                                  /* tp_methods */
     0,                                  /* tp_members */
-    0,                                  /* tp_getset */
+    glyph_getset,                       /* tp_getset */
     0,                                  /* tp_base */
     0,                                  /* tp_dict */
     0,                                  /* tp_descr_get */
